/*
	mmc.h: Deklarationen fr die Kommunikation mit der MMC (siehe mmc.c),
	      sowie fr das EEPROM.
	
 Die MMC wird im SPI-Modus verwendet (siehe auch mmc.c).
	

Pin configuration at MSP430F149:
--------------------------------
  MC			MC Pin          MMC         	MMC Pin
  P5.4 			48 		ChipSelect	1
  P5.2 / SlaveInMasterOut 46		DataIn 		2
  . 					GND 		3 (0 V)
  .					VDD 		4 (3.3 V)
  P5.3 UCLK1 / SlaveCLocK 47		Clock 		5
  .					GND 		6 (0 V)
  P5.1 / SlaveOutMasterIn 45		DataOut 	7
---------------------------------------------------------------------
 Revisions
 Date         	Author            			Revision
 5/2003		Rolf Freitag				0.02

*/

#ifndef _MMCLIB_H
#define _MMCLIB_H

// Defines:

// #define NULL 0x00

#ifndef TXEPT			// transmitter-empty flag
#define TEXPT 0x01
#endif

// macro defines
#define HIGH(a) ((a>>8)&0xFF)	// high byte from word
#define LOW(a) (a&0xFF)		// low byte from word


#define EN_LOW() P5OUT &= ~0x10	// Card Select
#define EN_HIGH() P5OUT |= 0x10	// Card Deselect
#define SPI_RXC (IFG2 & URXIFG1)
#define SPI_TXC (IFG2 & UTXIFG1)

#define SPI_RX_COMPLETE (IFG2 & URXIFG1)
#define SPI_TX_READY (IFG2 & UTXIFG1)

#define DUMMY 0xff


// Tokens (nessisary because at nop/idle (and CS active) only 0xff is on the data/command line)
#define MMC_START_DATA_BLOCK_TOKEN 		0xfe	// Data token start byte, Start Single Block Read
#define MMC_START_DATA_MULTIPLE_BLOCK_READ 	0xfe	// Data token start byte, Start Multiple Block Read
#define MMC_START_DATA_BLOCK_WRITE		0xfe	// Data token start byte, Start Single Block Write
#define MMC_START_DATA_MULTIPLE_BLOCK_WRITE	0xfc	// Data token start byte, Start Multiple Block Write
#define MMC_STOP_DATA_MULTIPLE_BLOCK_WRITE	0xfd	// Data toke stop byte, Stop Multiple Block Write


// an affirmative R1 response (no errors)
#define MMC_R1_RESPONSE 0x00


// this variable will be used to track the current block length
// this allows the block length to be set only when needed 
// unsigned long _BlockLength = 0;

// error/success codes
#define MMC_SUCCESS 0x00
#define MMC_BLOCK_SET_ERROR 0x01
#define MMC_RESPONSE_ERROR 0x02
#define MMC_DATA_TOKEN_ERROR 0x03
#define MMC_INIT_ERROR 0x04
#define MMC_CRC_ERROR 0x10
#define MMC_WRITE_ERROR 0x11
#define MMC_OTHER_ERROR 0x12
#define MMC_TIMEOUT_ERROR 0xFF


// commands: first bit 0 (start bit), second 1 (transmission bit); CMD-number + 0ffsett 0x40
#define MMC_GO_IDLE_STATE 	0x40
#define MMC_SEND_OP_COND 	0x41
#define MMC_READ_CSD 		0x49	//SEND_CSD
#define MMC_SEND_CID 		0x4a
#define MMC_STOP_TRANSMISSION 	0x4c
#define MMC_SEND_STATUS 	0x4d
#define MMC_SET_BLOCKLEN 	0x50	// Set block length for next read/write
#define MMC_READ_SINGLE_BLOCK 	0x51	// Read block from memory
#define MMC_READ_MULTIPLE_BLOCK 0x52
#define MMC_CMD_WRITEBLOCK 	0x54	// Write block to memory
#define MMC_WRITE BLOCK 	0x58
#define MMC_WRITE_MULTIPLE_BLOCK 0x59
#define MMC_WRITE_CSD 		0x5b	// PROGRAM_CSD
#define MMC_SET_WRITE_PROT 	0x5c
#define MMC_CLR_WRITE_PROT 	0x5d
#define MMC_SEND_WRITE_PROT 	0x5e
#define MMC_TAG_SECTOR_START 	0x60
#define MMC_TAG_SECTOR_END 	0x61
#define MMC_UNTAG_SECTOR 	0x62
#define MMC_TAG_EREASE_GROUP_START 0x63
#define MMC_TAG_EREASE_GROUP_END 0x64
#define MMC_UNTAG_EREASE_GROUP 	0x65
#define MMC_EREASE 		0x66
#define MMC_READ_OCR 		0x67
#define MMC_CRC_ON_OFF 		0x68


// setup uart1 in spi mode
void uart1_spi_mode_init (void);


// SPI-Ausgabe
void spi_write (const char);


// SPI-Einlesen
char spi_read (void);


// MMC-Blocklaenge setzen
char mmc_set_block_length (const unsigned int);


// Kommando zur MMC
void mmc_send_command (const char cmd, const char arg1, const char
		       arg2, const char arg3, const char arg4,
		       const char crc);


// Warten auf MMC-Antwort
char mmc_wait_for_response (const char);


// Initialisieren
char mmc_init (void);


// Lesen des size Byte grossen Blocks, der bei der angegebenen Adresse beginnt. Blockgroesse: count=2^n Byte
char mmc_read_block (const unsigned long int address,
		     const unsigned int count);

// Schreiben des 512 Byte grossen Blocks auf die MMC, der bei der angegebenen Adresse beginnt.
char mmc_write_block (const unsigned long int address);

// Register arg1 der Laenge arg2 auslesen (in den Buffer)
char mmc_read_register (const char, const unsigned char);

// Regiter arg1 der Laenge arg2 beschreiben (aus dem Buffer)
char mmc_write_register (const char, const unsigned char);

#endif /* _MMCLIB_H */
